#include "stm32g0xx.h"

#define SYS_TICK_PRIO   0

#if !defined  (HSE_VALUE)
#define HSE_VALUE    (8000000UL)    /*!< Value of the External oscillator in Hz */
#endif /* HSE_VALUE */

#if !defined  (HSI_VALUE)
  #define HSI_VALUE  (16000000UL)   /*!< Value of the Internal oscillator in Hz*/
#endif /* HSI_VALUE */

#if !defined  (LSI_VALUE)
 #define LSI_VALUE   (32000UL)     /*!< Value of LSI in Hz*/
#endif /* LSI_VALUE */

#if !defined  (LSE_VALUE)
  #define LSE_VALUE  (32768UL)      /*!< Value of LSE in Hz*/
#endif /* LSE_VALUE */

/*!< Uncomment the following line if you need to relocate your vector Table in
     Internal SRAM. */
/* #define VECT_TAB_SRAM */
#define VECT_TAB_OFFSET  0x0U /*!< Vector Table base offset field.
                                   This value must be a multiple of 0x100. */
/* The SystemCoreClock variable is updated in three ways:
  1) by calling CMSIS function SystemCoreClockUpdate()
  2) by calling HAL API function HAL_RCC_GetHCLKFreq()
  3) each time HAL_RCC_ClockConfig() is called to configure the system clock frequency
	 Note: If you use this function to configure the system clock; then there
		   is no need to call the 2 first functions listed above, since SystemCoreClock
		   variable is updated automatically.
*/
uint32_t SystemCoreClock = 16000000UL;

const uint32_t AHBPrescTable[16UL] = {0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 1UL, 2UL, 3UL, 4UL, 6UL, 7UL, 8UL, 9UL};
const uint32_t APBPrescTable[8UL] =  {0UL, 0UL, 0UL, 0UL, 1UL, 2UL, 3UL, 4UL};


void SystemCoreClockUpdate(void)
{
    uint32_t tmp;
    uint32_t pllvco;
    uint32_t pllr;
    uint32_t pllsource;
    uint32_t pllm;
    uint32_t hsidiv;

    switch (RCC->CFGR & RCC_CFGR_SWS)
    {
        case RCC_CFGR_SWS_0:                /* HSE used as system clock */
            SystemCoreClock = HSE_VALUE;
            break;
        case (RCC_CFGR_SWS_1 | RCC_CFGR_SWS_0):  /* LSI used as system clock */
            SystemCoreClock = LSI_VALUE;
            break;
        case RCC_CFGR_SWS_2:                /* LSE used as system clock */
            SystemCoreClock = LSE_VALUE;
            break;
        case RCC_CFGR_SWS_1:  /* PLL used as system clock */
            /* PLL_VCO = (HSE_VALUE or HSI_VALUE / PLLM) * PLLN
               SYSCLK = PLL_VCO / PLLR */
            pllsource = (RCC->PLLCFGR & RCC_PLLCFGR_PLLSRC);
            pllm = ((RCC->PLLCFGR & RCC_PLLCFGR_PLLM) >> RCC_PLLCFGR_PLLM_Pos) + 1UL;

            if(pllsource == 0x03UL) {
                /* HSE used as PLL clock source */
                pllvco = (HSE_VALUE / pllm);
            }
            else {
                /* HSI used as PLL clock source */
                pllvco = (HSI_VALUE / pllm);
            }
            pllvco = pllvco * ((RCC->PLLCFGR & RCC_PLLCFGR_PLLN) >> RCC_PLLCFGR_PLLN_Pos);
            pllr = (((RCC->PLLCFGR & RCC_PLLCFGR_PLLR) >> RCC_PLLCFGR_PLLR_Pos) + 1UL);

            SystemCoreClock = pllvco/pllr;
            break;
        case 0x00000000U:                   /* HSI used as system clock */
        default:                            /* HSI used as system clock */
            hsidiv = (1UL << ((RCC->CR & RCC_CR_HSIDIV) >> RCC_CR_HSIDIV_Pos));
            SystemCoreClock = (HSI_VALUE/hsidiv);
            break;
    }
    /* Compute HCLK clock frequency --------------------------------------------*/
    /* Get HCLK prescaler */
    tmp = AHBPrescTable[((RCC->CFGR & RCC_CFGR_HPRE) >> RCC_CFGR_HPRE_Pos)];
    /* HCLK clock frequency */
    SystemCoreClock >>= tmp;
}

void SystemClock_Config(void)
{
    /* Modify voltage scaling range */
    uint32_t tmp = PWR->CR1;
    tmp &= ~(PWR_CR1_VOS);
    tmp |= PWR_CR1_VOS_0;   //Voltage scaling range 1
    PWR->CR1 = tmp;
    /* Wait until VOSF is reset */
    while((PWR->SR2 & PWR_SR2_VOSF) == PWR_SR2_VOSF);

    /* HSI clock config */
    tmp = RCC->ICSCR;
    tmp &= ~(RCC_ICSCR_HSITRIM);
    tmp |= (64U << RCC_ICSCR_HSITRIM_Pos);  // Default HSI calibration trimming value
    RCC->ICSCR = tmp;
    /* Adjust the HSI16 division factor */
    tmp = RCC->CR;
    tmp &= ~(RCC_CR_HSIDIV);    // HSI clock is not divided
    RCC->CR = tmp;
    /* Update the SystemCoreClock global variable with HSISYS value  */
    SystemCoreClock = (HSI_VALUE / (1UL << ((RCC->CR & RCC_CR_HSIDIV) >> RCC_CR_HSIDIV_Pos)));

    /* Adapt Systick interrupt period */
    SysTick_Config(SystemCoreClock / 1000U);    // 1kHz
    NVIC_SetPriority(SysTick_IRQn, SYS_TICK_PRIO);

    /* LSI config */
    /* Disable the Internal Low Speed oscillator (LSI). */
    RCC->CSR &= ~(RCC_CSR_LSION);
    /* Wait till LSI is disabled */
    while ((RCC->CSR & RCC_CSR_LSIRDY) != 0U);

    /* PLL config */
    /* Disable the main PLL. */
    RCC->CR &= ~(RCC_CR_PLLON);
    /* Wait till PLL is ready */
    while ((RCC->CR & RCC_CR_PLLRDY) != 0U);
    /* Configure the main PLL clock source, multiplication and division factors. */
    tmp = RCC->PLLCFGR;
    tmp &= ~(RCC_PLLCFGR_PLLSRC | RCC_PLLCFGR_PLLM | RCC_PLLCFGR_PLLN | RCC_PLLCFGR_PLLP | RCC_PLLCFGR_PLLQ | RCC_PLLCFGR_PLLR);
    tmp |= RCC_PLLCFGR_PLLSRC_HSI;  // HSI clock selected as PLL entry clock source
    tmp |= (8 << RCC_PLLCFGR_PLLN_Pos);
    tmp |= RCC_PLLCFGR_PLLP_0;      // PLLP division factor = 2
    tmp |= RCC_PLLCFGR_PLLQ_0;      // PLLQ division factor = 2
    tmp |= RCC_PLLCFGR_PLLR_0;      // PLLR division factor = 2
    RCC->PLLCFGR = tmp;

    /* Enable the main PLL. */
    RCC->CR |= RCC_CR_PLLON;
    /* Enable PLLR Clock output. */
    RCC->PLLCFGR |= RCC_PLLCFGR_PLLREN      /*!< PLLRCLK selection from main PLL */;
    /* Wait till PLL is ready */
    while ((RCC->CR & RCC_CR_PLLRDY) == 0U);

    /* Program the new number of wait states to the LATENCY bits in the FLASH_ACR register */
    tmp = FLASH->ACR;
    tmp &= ~(FLASH_ACR_LATENCY);
    tmp |= FLASH_ACR_LATENCY_1; // FLASH Two wait states
    FLASH->ACR = tmp;
    /* Check that the new number of wait states is taken into account to access the Flash
       memory by polling the FLASH_ACR register */
    while ((FLASH->ACR & FLASH_ACR_LATENCY) != FLASH_ACR_LATENCY_1 /*!< FLASH Two wait states */);

    /* HCLK config */
    /* Set the highest APB divider in order to ensure that we do not go through
       a non-spec phase whatever we decrease or increase HCLK. */
    tmp = RCC->CFGR;
    tmp &= ~(RCC_CFGR_PPRE);
    tmp |= (RCC_CFGR_PPRE_2 | RCC_CFGR_PPRE_1 | RCC_CFGR_PPRE_0);   // HCLK divided by 16
    RCC->CFGR = tmp;

    /* Set the new HCLK clock divider */
    tmp = RCC->CFGR;
    tmp &= ~(RCC_CFGR_HPRE);    // SYSCLK not divided
    RCC->CFGR = tmp;

    /* SYSCLK config */
    tmp = RCC->CFGR;
    tmp &= ~(RCC_CFGR_SW);
    tmp |= RCC_CFGR_SW_1;   // PLL selection as system clock
    RCC->CFGR = tmp;
    while ((RCC->CFGR & RCC_CFGR_SWS) != (RCC_CFGR_SW_1 << RCC_CFGR_SWS_Pos));

    /* PCLK1 Configuration */
    tmp = RCC->CFGR;
    tmp &= ~(RCC_CFGR_PPRE);    // HCLK not divided
    RCC->CFGR = tmp;

    /* Adapt Systick interrupt period */
    SystemCoreClock = 64000000;
    SysTick_Config(SystemCoreClock / 1000U);    // 1kHz
    NVIC_SetPriority(SysTick_IRQn, SYS_TICK_PRIO);

#ifdef STM32G071xx
    /* Configure the USART2 clock source */
    tmp = RCC->CCIPR;
    tmp &= ~(RCC_CCIPR_USART2SEL);  // APB clock selected as USART2 clock
    RCC->CCIPR = tmp;
#endif
}

void SystemInit(void)
{
#ifdef VECT_TAB_SRAM
    SCB->VTOR = SRAM_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal SRAM */
#else
    SCB->VTOR = FLASH_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal FLASH */
#endif

    FLASH->ACR |= FLASH_ACR_PRFTEN;
    SysTick_Config(SystemCoreClock / 1000U);    // 1kHz
    NVIC_SetPriority(SysTick_IRQn, SYS_TICK_PRIO);

    RCC->APBENR2 |= RCC_APBENR2_SYSCFGEN;
    RCC->APBENR1 |= RCC_APBENR1_PWREN;

#ifdef STM32G071xx
    /* Change strobe configuration of GPIO depending on UCPDx dead battery settings */
    uint32_t tmp = SYSCFG->CFGR1;
    tmp &= ~(SYSCFG_CFGR1_UCPD1_STROBE | SYSCFG_CFGR1_UCPD2_STROBE);
    tmp |= (SYSCFG_CFGR1_UCPD1_STROBE | SYSCFG_CFGR1_UCPD2_STROBE);
    SYSCFG->CFGR1 = tmp;
#endif

    SystemClock_Config();
}
